@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -63,12 +35,16 @@ import com.openexchange.authorization.AuthorizationService;
 import com.openexchange.caching.CacheService;
 import com.openexchange.capabilities.CapabilityService;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.ForcedReloadable;
+import com.openexchange.config.Reloadable;
 import com.openexchange.config.cascade.ConfigViewFactory;
+import com.openexchange.config.lean.LeanConfigurationService;
 import com.openexchange.context.ContextService;
 import com.openexchange.crypto.CryptoService;
 import com.openexchange.database.CreateTableService;
 import com.openexchange.database.DatabaseService;
 import com.openexchange.dispatcher.DispatcherPrefixService;
+import com.openexchange.exception.OXException;
 import com.openexchange.groupware.delete.DeleteListener;
 import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.groupware.update.DefaultUpdateTaskProviderService;
@@ -84,6 +60,7 @@ import com.openexchange.oauth.provider.authorizationserver.grant.GrantManagement
 import com.openexchange.oauth.provider.authorizationserver.spi.OAuthAuthorizationService;
 import com.openexchange.oauth.provider.impl.DefaultAuthorizationService;
 import com.openexchange.oauth.provider.impl.OAuthProviderConstants;
+import com.openexchange.oauth.provider.impl.OAuthProviderMode;
 import com.openexchange.oauth.provider.impl.OAuthProviderProperties;
 import com.openexchange.oauth.provider.impl.OAuthResourceServiceImpl;
 import com.openexchange.oauth.provider.impl.authcode.AbstractAuthorizationCodeProvider;
@@ -104,6 +81,9 @@ import com.openexchange.oauth.provider.impl.groupware.CreateOAuthGrantTableServi
 import com.openexchange.oauth.provider.impl.groupware.CreateOAuthGrantTableTask;
 import com.openexchange.oauth.provider.impl.groupware.OAuthGrantConvertUtf8ToUtf8mb4Task;
 import com.openexchange.oauth.provider.impl.groupware.OAuthProviderDeleteListener;
+import com.openexchange.oauth.provider.impl.introspection.OAuthIntrospectionAuthorizationService;
+import com.openexchange.oauth.provider.impl.jwt.OAuthJWTScopeService;
+import com.openexchange.oauth.provider.impl.jwt.OAuthJwtAuthorizationService;
 import com.openexchange.oauth.provider.impl.servlets.AuthorizationEndpoint;
 import com.openexchange.oauth.provider.impl.servlets.RevokeEndpoint;
 import com.openexchange.oauth.provider.impl.servlets.TokenEndpoint;
@@ -112,7 +92,8 @@ import com.openexchange.oauth.provider.impl.servlets.TokenIntrospection;
 import com.openexchange.oauth.provider.resourceserver.OAuthResourceService;
 import com.openexchange.oauth.provider.resourceserver.scope.OAuthScopeProvider;
 import com.openexchange.osgi.HousekeepingActivator;
-import com.openexchange.osgi.SimpleRegistryListener;
+import com.openexchange.osgi.RankingAwareNearRegistryServiceTracker;
+import com.openexchange.osgi.service.http.HttpServices;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.serverconfig.ServerConfigService;
 import com.openexchange.sessiond.SessiondService;
@@ -139,9 +120,9 @@ public final class OAuthProviderActivator extends HousekeepingActivator {
 
     @Override
     protected Class<?>[] getNeededServices() {
-        return new Class<?>[] { DatabaseService.class, ConfigurationService.class, ContextService.class, UserService.class,
+        return new Class<?>[] { DatabaseService.class, LeanConfigurationService.class, ConfigurationService.class, ContextService.class, UserService.class,
             HttpService.class, DispatcherPrefixService.class, CryptoService.class, CacheService.class, ServerConfigService.class,
- SessiondService.class, CapabilityService.class, ConfigViewFactory.class, NotificationMailFactory.class, HazelcastConfigurationService.class };
+            SessiondService.class, CapabilityService.class, ConfigViewFactory.class, NotificationMailFactory.class, HazelcastConfigurationService.class };
     }
 
     @Override
@@ -149,9 +130,8 @@ public final class OAuthProviderActivator extends HousekeepingActivator {
         final ServiceLookup serviceLookup = this;
         Services.setServiceLookup(serviceLookup);
 
-        ConfigurationService configService = getService(ConfigurationService.class);
-        boolean providerEnabled = configService.getBoolProperty(OAuthProviderProperties.ENABLED, false);
-        if (!providerEnabled) {
+        LeanConfigurationService leanConfigService = getServiceSafe(LeanConfigurationService.class);
+        if (!leanConfigService.getBooleanProperty(OAuthProviderProperties.ENABLED)) {
             LOG.info("OAuth provider is disabled by configuration.");
             return;
         }
@@ -164,40 +144,41 @@ public final class OAuthProviderActivator extends HousekeepingActivator {
         trackService(MailService.class);
         track(OAuthScopeProvider.class, new OAuthScopeProviderTracker(context));
 
-        boolean isAuthorizationServer = configService.getBoolProperty(OAuthProviderProperties.IS_AUTHORIZATION_SERVER, true);
-        if (isAuthorizationServer) {
-            if ("hz".equalsIgnoreCase(configService.getProperty(OAuthProviderProperties.AUTHCODE_TYPE, "hz").trim())) {
-                final HazelcastConfigurationService hzConfigService = getService(HazelcastConfigurationService.class);
-                if (!hzConfigService.isEnabled()) {
-                    String msg = "OAuth provider is configured to use Hazelcast, but Hazelcast is disabled as per configuration! Aborting start of OAuth provider!";
-                    LOG.error(msg, new Exception(msg));
-                    return;
+        OAuthProviderMode mode = OAuthProviderMode.getProviderMode(leanConfigService.getProperty(OAuthProviderProperties.MODE));
+        switch (mode) {
+            default:
+            case AUTH_SEVER:
+                if ("hz".equalsIgnoreCase(leanConfigService.getProperty(OAuthProviderProperties.AUTHCODE_TYPE).trim())) {
+                    final HazelcastConfigurationService hzConfigService = getService(HazelcastConfigurationService.class);
+                    if (!hzConfigService.isEnabled()) {
+                        String msg = "OAuth provider is configured to use Hazelcast, but Hazelcast is disabled as per configuration! Aborting start of OAuth provider!";
+                        LOG.error(msg, new Exception(msg));
+                        return;
+                    }
+                    registerService(CustomPortableFactory.class, new PortableAuthCodeInfoFactory());
+
+                    String hzMapName = discoverHzMapName(hzConfigService.getConfig(), HzAuthorizationCodeProvider.HZ_MAP_NAME, LOG);
+                    track(HazelcastInstance.class, new HzTracker(hzMapName));
+                } else {
+                    startAuthorizationServer(new DbAuthorizationCodeProvider(this));
                 }
-
-                registerService(CustomPortableFactory.class, new PortableAuthCodeInfoFactory());
-
-                String hzMapName = discoverHzMapName(hzConfigService.getConfig(), HzAuthorizationCodeProvider.HZ_MAP_NAME, LOG);
-                track(HazelcastInstance.class, new HzTracker(hzMapName));
-            } else {
-                startAuthorizationServer(new DbAuthorizationCodeProvider(this));
-            }
+                break;
+            case EXPECT_JWT:
+                startJWTService(leanConfigService);
+                break;
+            case TOKEN_INTROSPECTION:
+                startTokenIntrospectionService(leanConfigService);
+                break;
         }
 
-        track(OAuthAuthorizationService.class, new SimpleRegistryListener<OAuthAuthorizationService>() {
+        RankingAwareNearRegistryServiceTracker<OAuthAuthorizationService> oAuthAuthorizationService = new RankingAwareNearRegistryServiceTracker<OAuthAuthorizationService>(context, OAuthAuthorizationService.class);
 
-            @Override
-            public void added(ServiceReference<OAuthAuthorizationService> ref, OAuthAuthorizationService service) {
-                registerService(OAuthResourceService.class, new OAuthResourceServiceImpl(service, serviceLookup));
-            }
-
-            @Override
-            public void removed(ServiceReference<OAuthAuthorizationService> ref, OAuthAuthorizationService service) {
-                unregisterService(service);
-            }
-
-        });
+        rememberTracker(oAuthAuthorizationService);
 
         openTrackers();
+
+        registerService(OAuthResourceService.class, new OAuthResourceServiceImpl(oAuthAuthorizationService, serviceLookup));
+
     }
 
     @Override
@@ -269,6 +250,24 @@ public final class OAuthProviderActivator extends HousekeepingActivator {
         }
     }
 
+    private void startJWTService(LeanConfigurationService leanConfigService) throws OXException {
+    	OAuthJWTScopeService scopeService = new OAuthJWTScopeService(leanConfigService);
+    	registerService(Reloadable.class, scopeService);
+
+        OAuthJwtAuthorizationService jwtAuthorizationService = new OAuthJwtAuthorizationService(leanConfigService, scopeService);
+        registerService(OAuthAuthorizationService.class, jwtAuthorizationService);
+        registerService(ForcedReloadable.class, jwtAuthorizationService);
+    }
+
+    private void startTokenIntrospectionService(LeanConfigurationService leanConfigService) {
+        OAuthJWTScopeService scopeService = new OAuthJWTScopeService(leanConfigService);
+        registerService(Reloadable.class, scopeService);
+
+        OAuthIntrospectionAuthorizationService tokenIntrospectionAuthorizationService = new OAuthIntrospectionAuthorizationService(leanConfigService, scopeService);
+        registerService(OAuthAuthorizationService.class, tokenIntrospectionAuthorizationService, null);
+        registerService(Reloadable.class, tokenIntrospectionAuthorizationService);
+    }
+
     private void stopAuthorizationServer() {
         ServiceReference<HttpService> httpServiceRef = context.getServiceReference(HttpService.class);
         if (httpServiceRef != null) {
@@ -277,11 +276,7 @@ public final class OAuthProviderActivator extends HousekeepingActivator {
                 if (httpService != null) {
                     while (!registeredServlets.isEmpty()) {
                         String alias = registeredServlets.pop();
-                        try {
-                            httpService.unregister(alias);
-                        } catch (Exception e) {
-                            LOG.error("Could not unregister servlet '{}'", alias, e);
-                        }
+                        HttpServices.unregister(alias, httpService);
                     }
                 }
             } finally {
@@ -333,6 +328,7 @@ public final class OAuthProviderActivator extends HousekeepingActivator {
             this.hzMapName = hzMapName;
         }
 
+        @SuppressWarnings("synthetic-access")
         @Override
         public HazelcastInstance addingService(ServiceReference<HazelcastInstance> reference) {
             HazelcastInstance hzInstance = context.getService(reference);
@@ -356,6 +352,7 @@ public final class OAuthProviderActivator extends HousekeepingActivator {
             // Nothing
         }
 
+        @SuppressWarnings("synthetic-access")
         @Override
         public void removedService(ServiceReference<HazelcastInstance> reference, HazelcastInstance service) {
             stopAuthorizationServer();
