@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -69,6 +41,7 @@ import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 import org.apache.commons.codec.binary.Base64;
 import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import com.google.common.net.HttpHeaders;
 import com.openexchange.ajax.LoginServlet;
 import com.openexchange.ajax.SessionUtility;
@@ -79,8 +52,7 @@ import com.openexchange.ajax.login.LoginTools;
 import com.openexchange.authentication.LoginExceptionCodes;
 import com.openexchange.authentication.LoginExceptionMessages;
 import com.openexchange.config.cascade.ConfigProviderService;
-import com.openexchange.config.cascade.ConfigView;
-import com.openexchange.config.cascade.ConfigViewFactory;
+import com.openexchange.config.lean.LeanConfigurationService;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextExceptionCodes;
@@ -148,9 +120,7 @@ public class AuthorizationEndpoint extends OAuthEndpoint {
     protected void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
         Tools.disableCaching(response);
         applyFrameOptions(response);
-        if (!Tools.considerSecure(request)) {
-            response.setHeader(HttpHeaders.LOCATION, URLHelper.getSecureLocation(request));
-            response.sendError(HttpServletResponse.SC_MOVED_PERMANENTLY);
+        if (isNotSecureEndpoint(request, response)) {
             return;
         }
 
@@ -205,9 +175,7 @@ public class AuthorizationEndpoint extends OAuthEndpoint {
     protected void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
         Tools.disableCaching(response);
         applyFrameOptions(response);
-        if (!Tools.considerSecure(request)) {
-            response.setHeader(HttpHeaders.LOCATION, URLHelper.getSecureLocation(request));
-            response.sendError(HttpServletResponse.SC_MOVED_PERMANENTLY);
+        if (isNotSecureEndpoint(request, response)) {
             return;
         }
 
@@ -324,7 +292,7 @@ public class AuthorizationEndpoint extends OAuthEndpoint {
         LoginServlet.addHeadersAndCookies(loginResult, response);
 
         // Add secret and public cookie
-        LoginServlet.writeSecretCookie(request, response, session, hash, true, serverName, loginConfig);
+        LoginServlet.writeSecretCookie(request, response, session, hash, Tools.considerSecure(request), serverName, loginConfig);
 
         return session;
     }
@@ -364,8 +332,7 @@ public class AuthorizationEndpoint extends OAuthEndpoint {
             User user = serverSession.getUser();
 
             // Check if OAuth is deactivated for this user
-            ConfigView configView = requireService(ConfigViewFactory.class, services).getView(user.getId(), context.getContextId());
-            if (!configView.opt(OAuthProviderProperties.ENABLED, Boolean.class, Boolean.TRUE).booleanValue() || user.isGuest()) {
+            if (!services.getServiceSafe(LeanConfigurationService.class).getBooleanProperty(user.getId(), context.getContextId(), OAuthProviderProperties.ENABLED) || user.isGuest()) {
                 return URLHelper.getErrorRedirectLocation(
                     authRequest.getRedirectURI(),
                     "access_denied",
@@ -657,7 +624,7 @@ public class AuthorizationEndpoint extends OAuthEndpoint {
         }
 
         try {
-            URI expectedReferer = new URI(URLHelper.getSecureLocation(request));
+            URI expectedReferer = new URI(URLHelper.getRequestLocation(request));
             URI actualReferer = new URI(referer);
             if (!stringsEqual(expectedReferer.getScheme(), actualReferer.getScheme())) {
                 return true;
@@ -674,7 +641,7 @@ public class AuthorizationEndpoint extends OAuthEndpoint {
             if (!stringsEqual(normalizePath(expectedReferer.getPath()), normalizePath(actualReferer.getPath()))) {
                 return true;
             }
-        } catch (URISyntaxException e) {
+        } catch (@SuppressWarnings("unused") URISyntaxException e) {
             return true;
         }
 
@@ -1181,6 +1148,7 @@ public class AuthorizationEndpoint extends OAuthEndpoint {
                 int intCode = Integer.parseInt(code);
                 return errorsByCodes.get(I(intCode));
             } catch (NumberFormatException e) {
+                LoggerFactory.getLogger(LoginError.class).debug("Code {} cannot be parsed as integer", code, e);
                 return null;
             }
         }
