@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -58,6 +30,7 @@ import java.io.Writer;
 import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Set;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
@@ -65,6 +38,9 @@ import org.json.JSONException;
 import org.json.JSONObject;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import com.google.common.collect.ImmutableSet;
+import com.google.common.net.HttpHeaders;
+import com.openexchange.config.ConfigurationService;
 import com.openexchange.exception.OXException;
 import com.openexchange.i18n.LocaleTools;
 import com.openexchange.i18n.Translator;
@@ -73,6 +49,7 @@ import com.openexchange.java.Strings;
 import com.openexchange.oauth.provider.authorizationserver.client.ClientManagement;
 import com.openexchange.oauth.provider.authorizationserver.grant.GrantManagement;
 import com.openexchange.oauth.provider.impl.OAuthProviderConstants;
+import com.openexchange.oauth.provider.impl.tools.URLHelper;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.templating.OXTemplate;
 import com.openexchange.templating.OXTemplateExceptionHandler;
@@ -91,6 +68,8 @@ public abstract class OAuthEndpoint extends HttpServlet {
 
     private static final long serialVersionUID = 6538319126816587520L;
 
+    private static final Set<String> URI_LOCALHOSTS = ImmutableSet.of("localhost", "127.0.0.1", "[::1]");
+
     protected static final String ATTR_OAUTH_CSRF_TOKEN = "oauth-csrf-token";
 
     protected final ClientManagement clientManagement;
@@ -109,6 +88,30 @@ public abstract class OAuthEndpoint extends HttpServlet {
         this.services = services;
     }
 
+    /**
+     * Checks whether given request was made using HTTPS or if plain HTTP is allowed for this request.
+     *
+     * @param request The HTTP request
+     * @return <code>true</code> if an insecure request was made and this is not allowed
+     */
+    private boolean isInsecureButMustNot(HttpServletRequest request) {
+        return !Tools.considerSecure(request);
+    }
+
+    /**
+     * Gets whether insecure requests (using plain HTTP) are allowed on localhost.
+     *
+     * @return <code>true</code> if so
+     */
+    protected boolean allowInsecureLocalhost() {
+        ConfigurationService configService = services.getService(ConfigurationService.class);
+        if (configService != null) {
+            return configService.getBoolProperty("com.openexchange.oauth.provider.allowInsecureLocalhost", false);
+        }
+
+        return false;
+    }
+
     /**
      * Responds with a HTML error page.
      *
@@ -188,7 +191,7 @@ public abstract class OAuthEndpoint extends HttpServlet {
         return locale;
     }
 
-    private void writeErrorPage(Writer writer, String message, Locale locale) throws OXException, IOException {
+    private void writeErrorPage(Writer writer, String message, Locale locale) throws OXException {
         TranslatorFactory translatorFactory = requireService(TranslatorFactory.class, services);
         TemplateService templateService = requireService(TemplateService.class, services);
         Translator translator = translatorFactory.translatorFor(locale);
@@ -206,4 +209,36 @@ public abstract class OAuthEndpoint extends HttpServlet {
         loginPage.process(vars, writer);
     }
 
+    /**
+     * Checks whether the request is allowed or not and sends an appropriate response in case it is not.
+     *
+     * @param request The request
+     * @param response The response
+     * @return <code>true</code> if the request is allowed, <code>false</code> otherwise
+     * @throws IOException
+     */
+    protected boolean isNotSecureEndpoint(HttpServletRequest request, HttpServletResponse response) throws IOException {
+        if (isInsecureButMustNot(request)) {
+
+            if (allowInsecureLocalhost()) {
+                String hostname = URLHelper.getHostname(request);
+                int portIndex = hostname.indexOf(':');
+                if (portIndex > 0) {
+                    hostname = hostname.substring(0, portIndex);
+                }
+
+                if (URI_LOCALHOSTS.contains(hostname)) {
+                    return false;
+                }
+                response.sendError(HttpServletResponse.SC_FORBIDDEN, "Localhost not allowed. Please adjust com.openexchange.oauth.provider.allowInsecureLocalhost");
+                return true;
+            }
+
+            response.setHeader(HttpHeaders.LOCATION, URLHelper.getSecureLocation(request));
+            response.sendError(HttpServletResponse.SC_MOVED_PERMANENTLY);
+            return true;
+        }
+        return false;
+    }
+
 }
